//------------------------------------------------------------------------------
// Lamp : Open source game middleware
// Copyright (C) 2004  Junpei Ohtani ( Email : junpee@users.sourceforge.jp )
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

/** @file
 * ストリングトークナイザヘッダ
 * @author Junpee
 */

#ifndef STRING_TOKENIZER_H_
#define STRING_TOKENIZER_H_

namespace Lamp{

//------------------------------------------------------------------------------
/**
 * ストリングトークナイザ
 */
class StringTokenizer{
public:
	/**
	 * コンストラクタ
	 *
	 * このコンストラクタを使用した場合、initialize()メソッドを呼ぶ必要があります。
	 */
	explicit StringTokenizer();
	 
	/**
	 * コンストラクタ
	 * @param target 解析される文字列
	 */
	explicit StringTokenizer(const String& target){
		initialize(target);
	}

	/**
	 * コンストラクタ
	 *
	 * このコンストラクタを使用した場合はStringTokenizerが破棄されるまで
	 * 解析されるデータを変更しないで下さい。
	 * @param target 解析されるデータ
	 */
	explicit StringTokenizer(const char* target){
		initialize(target);
	}

	/**
	 * コンストラクタ
	 * @param target 解析される文字列
	 * @param delimiter 区切り文字
	 */
	StringTokenizer(const String& target, const String& delimiter){
		initialize(target, delimiter);
	}

	/**
	 * コンストラクタ
	 *
	 * このコンストラクタを使用した場合はStringTokenizerが破棄されるまで
	 * 解析されるデータを変更しないで下さい。
	 * @param target 解析されるデータ
	 * @param delimiter 区切り文字
	 */
	StringTokenizer(const char* target, const String& delimiter){
		initialize(target, delimiter);
	}

	/**
	 * コンストラクタ
	 * @param target 解析される文字列
	 * @param delimiter 区切り文字
	 * @param delimiterOutput trueなら区切り文字をトークンとして出力
	 */
	StringTokenizer(const String& target, const String& delimiter,
		bool delimiterOutput){
		initialize(target, delimiter, delimiterOutput);
	}

	/**
	 * コンストラクタ
	 *
	 * このコンストラクタを使用した場合はStringTokenizerが破棄されるまで
	 * 解析されるデータを変更しないで下さい。
	 * @param target 解析されるデータ
	 * @param delimiter 区切り文字
	 * @param delimiterOutput trueなら区切り文字をトークンとして出力
	 */
	StringTokenizer(const char* target,
		const String& delimiter, bool delimiterOutput){
		initialize(target, delimiter, delimiterOutput);
	}

	/**
	 * 初期化
	 * @param target 解析される文字列
	 * @param delimiter 区切り文字
	 * @param delimiterOutput trueなら区切り文字をトークンとして出力
	 */
	void initialize(const char* target,
		const String& delimiter = defaultDelimiter,
		bool delimiterOutput = false);

	/**
	 * 初期化
	 * @param target 解析される文字列
	 * @param delimiter 区切り文字
	 * @param delimiterOutput trueなら区切り文字をトークンとして出力
	 */
	void initialize(const String& target,
		const String& delimiter = defaultDelimiter,
		bool delimiterOutput = false);

	/**
	 * デストラクタ
	 */
	virtual ~StringTokenizer();

	/**
	 * トークンがまだあるかどうか
	 * @return trueならトークンがある。
	 */
	virtual bool hasMoreTokens();

	/**
	 * 次のトークン取得
	 * @return 次のトークン
	 */
	virtual String getNextToken();

	/**
	 * トークンオフセットの取得
	 * @return トークンオフセット
	 */
	virtual int getTokenOffset() const{ return tokenOffset_; }

	/**
	 * 現在のトークン取得
	 * @return 現在のトークン
	 */
	virtual String getNowToken() const{ return nowToken_; }

	/**
	 * 一つ前のトークン取得
	 * @return 一つ前のトークン
	 */
	virtual String getPreviousToken() const{ return previousToken_; }

private:

	// コピーコンストラクタの隠蔽
	StringTokenizer(const StringTokenizer& copy);

	// 代入コピーの隠蔽
	void operator =(const StringTokenizer& copy);

	// 対象文字列。文字列データの確保が目的。
	String targetString_;
	// ベースポインタ
	const char* base_;
	// カレントポインタ
	const char* current_;
	// トークンオフセット
	int tokenOffset_;
	// デリミタ
	String delimiter_;
	// 現在のトークン
	String nowToken_;
	// 一つ前のトークン
	String previousToken_;
	// デリミタ出力フラグ
	bool delimiterOutput_;

	// デフォルトデリミタ
	static const String defaultDelimiter;

};

//------------------------------------------------------------------------------
} // End of namespace Lamp
#endif // End of STRING_TOKENIZER_H_
//------------------------------------------------------------------------------
